#include <iostream>
#include <sstream>
#include <pthread.h>
#include <boost/algorithm/string.hpp>

#include <ros/ros.h>
#include <sensor_msgs/LaserScan.h>
#include <tf/transform_listener.h>

#include <move_base_msgs/MoveBaseAction.h>
#include <actionlib/client/simple_action_client.h>


#include <geometry_msgs/PoseWithCovarianceStamped.h>
#include <geometry_msgs/PoseStamped.h>

#include <LinearMath/btQuaternion.h>
#include <LinearMath/btMatrix3x3.h>

#include <TCPInterface/RCOMMessageReceived.h>
#include <TCPInterface/RCOMMessageToSend.h>

#include <TCPInterface/GenericAction.h>

#include "gnetwork/networkedmodule.h"

#include "CmdServer.h"
#include "TcpClient.h"


#include <std_msgs/String.h>

#define RAD2DEG(a) ((a)/M_PI*180.0)
#define DEG2RAD(a) ((a)*M_PI/180.0)

#define radians(a) ((a)/180.0*M_PI)

#include <fstream>
#include <set>
#include <string>


using namespace std;

 struct Place {
   Place(std::string tag_,
         float x_, float y_, float z_,
         float rx_, float ry_, float rth_){
     tag=tag_;
     x=x_;
     y=y_;
     z=z_;
     rx=rx_;
     ry=ry_;
     rth=rth_;
   }
   string tag;
   float x,y,z; // point in the robot frame
   float rx, ry, rth; // position of the robot in the map
   bool operator<(const Place& p) const {
     return tag<p.tag;
   }
 };

 typedef std::set<Place> PlaceSet;

 ostream& operator << (ostream& os, const Place& p){
   os << p.tag  << " " << p.x << " " << p.y << " " << p.z << " ";
   os << p.rx << " " << p.ry << " " << p.rth << endl;
   return os;
 }

 ostream& operator<<(ostream& os, const PlaceSet& ps){
   for (PlaceSet::const_iterator it=ps.begin(); it!=ps.end(); it++){
     os << *it;
   }
   return os;
 }

 istream& operator >> (istream& is, Place& p){
   is >> p.tag >> p.x >> p.y >> p.z >> p.rx >> p.ry >> p.rth;
   return is;
 }

 istream& operator >> (istream& is, PlaceSet& ps)
 {
   Place p("", 0., 0., 0., 0., 0., 0.);
   while(is)
   {
     is >> p;
     ps.insert(p);
  }
  return is;
 }
 
const Place* findPlace(const PlaceSet& ps, const std::string& s){
  Place k(s,0,0,0,0,0,0);
  PlaceSet::const_iterator it = ps.find(k);
  if (it == ps.end())
     return 0;
  return &(*it);
}

PlaceSet placeSet;

ros::Publisher pub1, pub2, pubmsg, speakyPub;
ros::NodeHandle *_n, *_np;

const char *topic_pose = "amcl_pose";      
const char *topic_goal = "move_base_simple/goal";
const char *topic_init = "initialpose";
// const char *topic_action = "tcp_action";
const char *topic_message_received = "RCOMmessageReceived";
const char *topic_message_to_send = "RCOMmessageReceived";
const char *speaky_info_to_send = "SpeakyInformation";



tf::TransformListener* transformListener=0;
std::string map_frame_id="map";
std::string laser_frame_id="laser_frame";
std::string base_link_frame_id="base_link";
std::string laser_topic="scan";

ros::Subscriber laserSubscriber;

volatile bool lockedString=false;
std::string lastRobotLaserString;
std::string lastRobotLaserStringOpenRDK;

struct RobotPose
{
      double x,y,theta;
};

static RobotPose robot_pose;
static int counter=0;
//static TCPInterface::TCAction a;
static actionlib::SimpleActionClient<move_base_msgs::MoveBaseAction> *ac;

static string robotname;

#define BUFLEN 2000
static int server_port;
static char bufin[BUFLEN], bufout[BUFLEN];

using namespace GNetwork;

struct UDPNetworkedSupplier: public NetworkedSupplier {
	virtual int parseSupplierMessage(unsigned int size, const char* buf);
};

int UDPNetworkedSupplier::parseSupplierMessage(unsigned int size, const char* buf){
	std::cout << "received [" << size << "] : " << buf << endl;
	// cout << "peers# " << peers.size() << endl;
	return size;
}

struct UDPNetworkClient: public NetworkedClient{
	virtual int parseMessage(unsigned int size, const char* buf);
	virtual void elapsedTimeoutReceived();
};

int UDPNetworkClient::parseMessage(unsigned int size, const char* buf){
	std::cout << "received [" << size << "] : " << buf << endl;
	return size;
}
void UDPNetworkClient::elapsedTimeoutReceived(){
	std::cout << "timeout elapsed" << endl;
	NetworkedClient::elapsedTimeoutReceived();
}

static	UDPNetworkedSupplier UDPsupplier;
static	UDPNetworkClient UDPclient;
static pthread_t udpth;
//static pthread_mutex_t mutex1 = PTHREAD_MUTEX_INITIALIZER;

int UDPServerPort=0;
int UDPMinPort=0, UDPMaxPort=0;
int ping_delay = 10; // sec.

void* UDP_discover_run(void *)
{
	// Read config file

	//ifstream f("config/UDPpeers.cfg");

	//f.close();

	for (int UDPClientPort=UDPMinPort; UDPClientPort<=UDPMaxPort; UDPClientPort++) {
		if (UDPClientPort==UDPServerPort) continue;
		PeerInfo pinfo;
		UDPclient.peers.insert(make_pair(Address::getAddressByName("localhost", UDPClientPort), pinfo));
	}
	if (UDPclient.bind(Address::getAddressByName("localhost", 0))==-1){
		cerr << "cannot bind UDP client socket" << endl;	
	}
	char buf[64];
	while (true) {
		usleep(ping_delay*1e6);
		cout << "ping around..." << endl;
		sprintf(buf,"hello %d",UDPServerPort);
		UDPclient.sendToAll(strlen(buf),buf);
	}
	return NULL;
}

void UDPStart() {

	if (UDPServerPort==0) return;

	// start server
	UDPsupplier.clientSuppressionTime=10000;
	if (UDPsupplier.bind(Address::getServerAddress(UDPServerPort))==-1){
		cerr << "cannot bind UDP server socket on port " << UDPServerPort << endl;	
		return;
	}
	UDPsupplier.start();
	cout << "UDP server running on port " << UDPServerPort << endl;

	// start thread for discovery clients
	pthread_create( &udpth, NULL, UDP_discover_run, NULL);

	return;

}



class ClientList
{
  public:
    
  vector<TcpClient*> clients;
  
  ClientList() {}
  
  ~ClientList() {
      for (size_t i=0; i<clients.size(); i++) {
          clients[i]->close();
          delete clients[i];
      }
  }
  
  TcpClient *findClient(string name) {
      TcpClient *r =NULL;
      for (size_t i=0; i<clients.size(); i++)  {
          if (clients[i]->getHostPort_string()==name)
	  return clients[i];
      }
      return r;
  }
  
  string to_string(string host, int port)
  {
      stringstream ss;
      ss << host << ":" << port;
      return ss.str();
  }
  
  bool connect(string host, int port) {
      bool b = true;
      TcpClient *c = findClient(to_string(host,port));
      if (c==NULL) {
        c = new TcpClient();
        clients.push_back(c);
      }
      if (!c->connected)
        b = c->connect(host,port);
      if (b && c->connected) {
          stringstream ss;
          ss << "hello " << robotname << " " << server_port << "\n\r";
          c->send(ss.str().c_str());
          usleep(100);
          c->receive(bufin,BUFLEN-1); // cout << bufin << endl;
      }
      return b;
  }
  
  bool send(string name, string buffer) {
      return send(name,buffer.c_str());
  }
  
  bool send(string name, const char* buffer) {
      std::vector<std::string> strs;
      boost::split(strs, name, boost::is_any_of(":"));
      if (strs.size()<2) return false;
      
      return send(strs[0],atoi(strs[1].c_str()),buffer);
  }


  bool send(string host, int port, const char* buffer) {
      bool b = true;
      TcpClient *c = findClient(to_string(host,port));
      if (c==NULL) {
          b = connect(host,port);
          c = findClient(to_string(host,port));
          if (c==NULL) b = false;
      }
      if (b && !c->connected) {
          b = connect(host,port);
      }
      if (b && c->connected) {
          b = c->send(buffer);  // cout << "send " << buffer << endl;
          usleep(100);
          c->receive(bufin,BUFLEN-1); // cout << bufin << endl;
      }
      else 
          b=false;
      return b;
  }

  
};

static ClientList clients;
static map<string,string> clientnames;

void deliver(const char * mess, const char *host, int port)
{
    TCPInterface::RCOMMessageReceived msg;
    msg.header.stamp = ros::Time::now();
    stringstream ss;
    ss << host << ":" << port;
    msg.robotsender = clientnames[ss.str()];
    msg.robotreceiver = "Me";
    msg.value = string(mess[0]==' '?(mess+1):mess);
    pubmsg.publish(msg);
}

bool send(const char * mess)
{
    stringstream ss(mess);
    string name;
    ss >> name;
    char buf[strlen(mess)];
    ss.read(buf,strlen(mess)-1);
    size_t r = ss.gcount();
    buf[r]='\0';
    //cout << "remaining: " << buf << endl;
    
    sprintf(bufout,"message %s\n\r",buf[0]==' '?(buf+1):buf);
    return clients.send(name,bufout);
}

void hello(const char * mess, const char *host, int port)
{  
    stringstream ss(mess);
    string name; int sport;
    ss >> name;
    ss >> sport;
    
    cout << "Received hello from " << name << "|" << host << ":" << sport << " ("<< host << ":" << port << ")" << endl;
    
    stringstream ks; ks << host << ":" << port;
    stringstream ns; ns << name << "|" << host << ":" << sport;
    
    // cout << "map[" << ks.str() <<"] = " << ns.str() << endl;
    clientnames[ks.str()] = ns.str();
    
    // cout << "map value now = " << clientnames[ks.str()] << endl;
    
    clients.connect(host,sport);
}


static actionlib::SimpleActionClient<TCPInterface::GenericAction> *gac = NULL;

void doAction(string action_str)
{  
    string name="null", params="";
    stringstream ss(action_str);
    ss >> name;
    if (action_str.size() > name.size()+2)
        params = string(action_str.substr(name.size()+2));

    if (gac==NULL) {
        gac = new actionlib::SimpleActionClient<TCPInterface::GenericAction>("generic",true);
    }
    
    while(!gac->waitForServer(ros::Duration(5.0))){
	ROS_INFO("Waiting for the generic_action action server to come up");
    }
    TCPInterface::GenericGoal goal;
    
    goal.id = counter++;
    goal.name = name;
    goal.params = params;
    goal.function = "start";
    // cout << "TCPInterface: doaction " << goal << endl;
    
    gac->sendGoal(goal);

}

void gotopose_simple(string args)
{
    float x,y,th;

    stringstream ss(args);
    ss >> x; ss >> y; ss >> th;
    th = DEG2RAD(th);

    geometry_msgs::PoseStamped goal;
    goal.header.frame_id = "/map";
    goal.header.stamp = ros::Time::now();
    goal.pose.position.x = x;
    goal.pose.position.y = y;
    goal.pose.orientation.z = sin(th/2);
    goal.pose.orientation.w = cos(th/2);
    
    cerr << "Target Pose:" << endl;
    cerr << goal << endl;
    pub1.publish(goal);  
}

void gotoposeArgs(const Place* p)
{
  geometry_msgs::PoseStamped goal;
  goal.header.frame_id = "/map";
  goal.header.stamp = ros::Time::now();
  goal.pose.position.x = p->rx;
  goal.pose.position.y = p->ry;
  goal.pose.orientation.z = sin(p->rth/2);
  goal.pose.orientation.w = cos(p->rth/2);
  
  cerr << "Target Pose:" << endl;
  cerr << goal << endl;
  pub1.publish(goal);  
}


void gotopose(string args)
{
    float x,y,th;

    stringstream ss(args);
    ss >> x; ss >> y; ss >> th;
    th = DEG2RAD(th);

    //wait for the action server to come up
    while(!ac->waitForServer(ros::Duration(5.0))){
	ROS_INFO("Waiting for the move_base action server to come up");
    }

    move_base_msgs::MoveBaseGoal goal;

  goal.target_pose.header.frame_id = "/map";
  goal.target_pose.header.stamp = ros::Time::now();
  goal.target_pose.pose.position.x = x;
  goal.target_pose.pose.position.y = y;
  goal.target_pose.pose.orientation.z = sin(th/2);
  goal.target_pose.pose.orientation.w = cos(th/2);

  ROS_INFO("Sending goal");
  ac->sendGoal(goal);
/*
  ac->waitForResult();   // blocking!

  if(ac->getState() == actionlib::SimpleClientGoalState::SUCCEEDED)
    ROS_INFO("Goto finished");
  else
    ROS_INFO("Goto failed");
*/
  return;
}

void stop()
{  
    //wait for the action server to come up
    while(!ac->waitForServer(ros::Duration(5.0))){
	ROS_INFO("Waiting for the move_base action server to come up");
    }
    ac->cancelAllGoals();
}

void initpose(string args)
{
    float x,y,th;

    stringstream ss(args);
    ss >> x; ss >> y; ss >> th;
    th = DEG2RAD(th);

    geometry_msgs::PoseWithCovarianceStamped init;
    init.header.frame_id = "/map";
    init.header.stamp = ros::Time::now();
    init.pose.pose.position.x = x;
    init.pose.pose.position.y = y;
    init.pose.pose.orientation.z = sin(th/2);
    init.pose.pose.orientation.w = cos(th/2);
    
    cerr << "Init Pose:" << endl;
    cerr << init << endl;
    pub2.publish(init);  
}

bool openrdkcmdgetparse(const char * in, char * out)
{
	//cerr << in << endl;
	sprintf(out,"ERROR");

	if (strcasecmp(in,"/robot/estimatedPose")==0) {
	    sprintf(out,"Value-of: %s %.2f %.2f %.2f",in,robot_pose.x,robot_pose.y,RAD2DEG(robot_pose.theta));	
	}
	else if (strcasecmp(in,"/laserData/out/laserStringOut")==0) {
	    strcpy(out, "Value-of: /laserData/out/laserStringOut ");
	    while (lockedString) usleep(1000);
	    lockedString = true;
	    strcat(out,lastRobotLaserStringOpenRDK.c_str());
	    lockedString = false;
	}
	strcat(out,"\n\r");
	return true;
}

bool openrdkcmdsetparse(const char * in, char * out)
{
      	cerr << in << endl;
      
	sprintf(out,"ERROR");
	if (strstr(in,"/robot/targetPose")) {	    
	    const char *str = in+strlen("/robot/targetPose");
	    gotopose(str);	    
	    sprintf(out,"OK");	
	}
	else if (strstr(in,"/localizer/in/initPose")) {	    
	    const char *str = in+strlen("/localizer/in/initPose");
	    initpose(str);
	    sprintf(out,"OK");	
	}
	else if (strstr(in,"/robot/desiredSpeed")) {	    
//	    const char *str = in+strlen("...");
//	    sprintf(out,"OK");
	}
	else if (strstr(in,"/robot/desiredJog")) {	    
	}
	strcat(out,"\n\r");
	return true;
}

void localizerCallback(const geometry_msgs::PoseWithCovarianceStamped::ConstPtr& msg)
{
    geometry_msgs::Quaternion q = msg->pose.pose.orientation;
    btQuaternion bq(q.x,q.y,q.z,q.w);
    double roll,pitch,yaw;
		tf::Matrix3x3(bq).getRPY(roll,pitch,yaw);
    robot_pose.x = msg->pose.pose.position.x;
    robot_pose.y = msg->pose.pose.position.y;
    robot_pose.theta = yaw;
    /*cout << "Time: " << msg->header.stamp.sec << "." << msg->header.stamp.nsec << endl;
    cout << "Frame: " << msg->header.frame_id << endl;
    cout << "Pose: " << msg->pose.pose.position.x << " " << msg->pose.pose.position.y << " ";
    cout << yaw/M_PI*180.0 << endl;*/
}

void laserCallback(const sensor_msgs::LaserScan::ConstPtr& scan)
{
  std::string frame_id = scan->header.frame_id;
  ros::Time t = scan->header.stamp;
  tf::StampedTransform laserPose, odomPose;
  std::string error;
  stringstream os,os2;
  lastRobotLaserString = ""; lastRobotLaserString = "";

  if (transformListener->waitForTransform (map_frame_id, laser_frame_id, t, ros::Duration(0.5), ros::Duration(0.01), &error)){
      transformListener->lookupTransform(map_frame_id, laser_frame_id, t, laserPose);
      transformListener->lookupTransform(map_frame_id, base_link_frame_id, t, odomPose);

    double odomX, odomY, odomTheta;
    double laserX, laserY, laserTheta;
    odomX=odomPose.getOrigin().x();
    odomY=odomPose.getOrigin().y();

    double yaw,pitch,roll;
		tf::Matrix3x3 mat =  odomPose.getBasis();
    mat.getEulerZYX(yaw, pitch, roll);
    odomTheta = yaw;

    laserX=laserPose.getOrigin().x();
    laserY=laserPose.getOrigin().y();
    mat =  laserPose.getBasis();
    mat.getEulerZYX(yaw, pitch, roll);
    laserTheta = yaw;

    os.setf(ios::fixed); 
    os << setprecision(6);
    os << "ROBOTLASER1 " << 0 << " "
       << scan->angle_min << " " 
       << scan->angle_max - scan->angle_min << " "
       << scan->angle_increment << " " 
       << scan->range_max << " "
       << 0.1 << " " // accuracy
       << 0 << " " // remission mode
       <<  scan->ranges.size() << " ";
    os << setprecision(2);
    for (uint i=0; i<scan->ranges.size(); i++){
      if (scan->ranges[i]<scan->range_min)
	os  << scan->range_max;
      else 
	os << scan->ranges[i];
      os << " ";
    }
    os << " 0 "; // 0 remissions
    os.setf(ios::fixed);
    os << setprecision(6);
    
    os << laserX << " " << laserY << " " << laserTheta << " ";
    os << odomX << " " << odomY << " " << odomTheta << " ";
    os << 0  << " " // laser tv
       << 0  << " " // laser rv
       << 0  << " " // forward_safety_dist
       << 0  << " " // side_safty_dist
       << 0  << " "; // turn_axis
    
    char timeString[50];
    sprintf(timeString, "%d.%08d", scan->header.stamp.sec, scan->header.stamp.nsec);
    os << timeString << " hostname ";
    os << timeString << endl;

    const int reduced=3;

    // OpenRDK format
    os2.setf(ios::fixed); os2 << setprecision(6);
    // laserPose (relative to robot) 
    os2 << 0.0 << " " << 0.0 << " " << 0.0 << " ";
    // beamsOfset 
    os2 << RAD2DEG(scan->angle_increment)*reduced << " ";
    // firstBeam
    os2 << RAD2DEG(scan->angle_min) << " ";
    // beamNumber
    os2 << scan->ranges.size()/reduced << " ";
    // range data
    os2 << setprecision(2);
    for (uint i=0; i<scan->ranges.size(); i+=reduced){
      if (scan->ranges[i]<scan->range_min)
	os2  << scan->range_max;
      else 
	os2 << scan->ranges[i];
      os2 << " ";
    }
    os2 << endl;

    while (lockedString) usleep(1000);
    lockedString = true;
    lastRobotLaserString = os.str();
    lastRobotLaserStringOpenRDK = os2.str();
    lockedString = false;
  } else {
    std::cerr << error << std::endl; 
  }
}

void RCOMMessageToSendCallback(const TCPInterface::RCOMMessageToSend::ConstPtr& msg)
{
    cout << "Message to send: " << endl <<
      msg->header <<
      msg->robotsender << endl <<
      msg->robotreceiver << endl <<
      msg->value << endl << endl;
      
    clients.send(msg->robotreceiver,msg->value);
}

bool cmdparse (const char * in, char * out, const char *host, int port) 
{
	sprintf(out,"Unknown command %s",in);
	char buf1[160],buf2[160];

	if (strncasecmp(in,"getvalue",8)==0)
		return openrdkcmdgetparse(in+9,out);
	else if (strncasecmp(in,"setvalue",8)==0)
		return openrdkcmdsetparse(in+9,out);
	
	std_msgs::String speakyString;
	speakyString.data = string(in);
	speakyPub.publish(speakyString);
	const Place* place=findPlace(placeSet, speakyString.data);
	cout << "*** received from " << host << ":" << port << " [" << in << "] ***" << endl;
	sprintf(out,"OK");

	if (place == 0)
	{
	  std::cerr << "Destination not found" << std::endl;
	}
	else
	{
	  gotoposeArgs(place);
	}
	
	switch (in[0]) {

	  case 'a': case 'A':
	    if (strncasecmp(in,"action",6)==0) {
	        const char *action_str = (in+6);
	        sprintf(out,"Exec action: %s",action_str);
	        cout << "Exec action: " << action_str << endl;
	        doAction(action_str);
	    }
	    break;
	    
	   case 'g': case 'G':
		if (strcasecmp(in,"getpose")==0) {
			sprintf(out,"pose %.2f %.2f %.2f",robot_pose.x,robot_pose.y,RAD2DEG(robot_pose.theta));
		}
		else if (strncasecmp(in,"gettopic",7)==0) {
			sscanf(in,"%s %s",buf1,buf2);
			printf("rostopic echo %s",buf2);
			sprintf(out,"%s = ...",buf2);
		}
		else if (strncasecmp(in,"getparam",7)==0) {
			sscanf(in,"%s %s",buf1,buf2);
			printf("rosparam get %s",buf2);
			sprintf(out,"%s = ...",buf2);
		}
		if (strncasecmp(in,"gotopose",8)==0) {
		      gotopose(in+8);
		      sprintf(out,"OK gotopose: %s",in+8);
		}
		break;
	  
	   case 'h': case 'H':
	      if (strncasecmp(in,"hello",5)==0) {
	          hello(in+5,host,port);
	          sprintf(out,"hello OK");
	      }
	      break;

	   case 'i': case 'I':
	      if (strncasecmp(in,"initpose",8)==0) {
		      initpose(in+8);
		      sprintf(out,"OK initpose: %s",in+8);
	      }
	      break;


	    case 'l': case 'L':
	        if (strcasecmp(in,"laser_subscribe")==0) {
		laserSubscriber = _n->subscribe(laser_topic, 100, laserCallback);
	        }
	        else if (strcasecmp(in,"laser_unsubscribe")==0) {
		laserSubscriber.shutdown();
	        }
	        break;

	  case 'm': case 'M':
		if (strncasecmp(in,"message",7)==0) {
		      deliver(in+7,host,port);
		      sprintf(out,"OK message");
		}
		break;

	  case 's': case 'S':
		if (strcasecmp(in,"stop")==0) {
		      sprintf(out,"OK stop");
		      stop();
		}
		else if (strncasecmp(in,"send",4)==0) {
		      bool b = send(in+4);
		      sprintf(out,"%s send",(b?"OK":"ERROR"));
		}
		break;
		
		
	    // printf(" -- %s\n",in);

   	  default:
	    sprintf(out,"Unknown command.");
	    break;
	  

	}
	strcat(out,"\n\r");
	return true;
}

int main(int argc, char** argv)
{
  int refresh_rate;

  ros::init(argc, argv, "TCPInterface");
  ros::NodeHandle n, np("~");
  _n = &n; _np = &np;
  string config_file="";

  ifstream ifs("/home/erratic/src/DIS-ros/TCPInterface/bin/places/giorgio_semantic_house.places");
  ifs >> placeSet;
  cerr << "Read all the places" << endl;
  cerr << placeSet << endl;

  // param("refresh_rate", refresh_rate, 10);
  
  if(!np.getParam("refresh_rate", refresh_rate))
  {
    refresh_rate=10; // Hz
  }
  if(!np.getParam("server_port", server_port))
  {
    server_port=9000;
  }
  if(!np.getParam("TCP_server_port", server_port))
  {
    server_port=9000;
  }
  if(!np.getParam("UDP_server_port", UDPServerPort))
  {
    UDPServerPort=0;
  }
  if(!np.getParam("config_file", config_file))
  {
    config_file="";
  }

  string tf_prefix="NONE";
  n.getParam("tf_prefix", tf_prefix);
  robotname = tf_prefix;
  cout << "robotname = " << robotname << endl;

  UDPStart();
  CmdServer server;
  server.setParseFn(cmdparse);
  server.open(server_port);
  server.start();
  ROS_INFO_STREAM("TCP Listening on port "<<server_port<<endl);

  np.getParam("map_frame_id", map_frame_id);
  np.getParam("base_link_frame_id", base_link_frame_id);
  np.getParam("laser_frame_id", laser_frame_id);
  np.getParam("laser_topic", laser_topic);
  cerr << "map_frame_id " << map_frame_id << endl;
  cerr << "base_link_frame_id " << base_link_frame_id << endl;
  cerr << "laser_frame_id " << laser_frame_id << endl;
  cerr << "laser_topic " << laser_topic << endl;
  transformListener = new tf::TransformListener;
  
  ros::Subscriber RCOMMessageSubscriber = n.subscribe(topic_message_to_send, 10, RCOMMessageToSendCallback);
  ros::Subscriber localizerSubscriber = n.subscribe(topic_pose, 10, localizerCallback);
  
  const int queue_size=100;
  
  pub1 = n.advertise<geometry_msgs::PoseStamped>(topic_goal,queue_size);
  pub2 = n.advertise<geometry_msgs::PoseWithCovarianceStamped>(topic_init,queue_size);
  // pubact = n.advertise<TCPInterface::TCPGenericAction>(topic_action,queue_size);
  pubmsg = n.advertise<TCPInterface::RCOMMessageReceived>(topic_message_received,queue_size);

  speakyPub = n.advertise<std_msgs::String>(speaky_info_to_send, queue_size);
  
  ac = new actionlib::SimpleActionClient<move_base_msgs::MoveBaseAction>("move_base", true);

  ros::Rate rate(refresh_rate);
  while(ros::ok())
  {
    ros::spinOnce();
    rate.sleep();
  }

  delete ac;

  return 0;
}
